#include <qcombobox.h>
#include <qspinbox.h>
#include <qstringlist.h>
#include <qlabel.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qlayout.h>

#ifdef QT4
#include <QGroupBox>
#else
#include <qvbox.h>
#include <qhbox.h>
#include <qvgroupbox.h>
#endif

#include "uiconfig.h"
#include "skinmanager.h"

UIConfig::UIConfig(Configuration &conf, QWidget *parent)
    :	ConfigView(conf, parent)
{
}

void UIConfig::setStatesByConfiguration(const Configuration &conf)
{
    ConfigView::setStatesByConfiguration(conf);

    changePlayInfoLayoutInPortaitMode_->setChecked(conf.changePlayInfoLayoutInPortaitMode);
    playlistAutoTriggerFilterInterval_->setValue(conf.playlistAutoTriggerFilterInterval);

    touchModeEnabled_->setChecked(conf.isTouchModeEnabled);
    kineticScrollingEnabled_->setChecked(conf.isKineticScrollingEnabled);

    // Set Skin
#ifdef QT4
    int index = skinList1->findText(conf.skin1);
    if (index > -1)
        skinList1->setCurrentIndex(index);

    index = skinList2->findText(conf.skin2);
    if (index > -1)
        skinList2->setCurrentIndex(index);
#else
    for (int i = 0; i != skinList1->count(); ++i)
        if (skinList1->text(i) == conf.skin1)
            skinList1->setCurrentItem(i);

    for (int i = 0; i != skinList2->count(); ++i)
        if (skinList2->text(i) == conf.skin2)
            skinList2->setCurrentItem(i);
#endif
}

void UIConfig::createWidgets()
{
    ConfigView::createWidgets();

    QVBoxLayout *layout = new QVBoxLayout(widget());
    layout->setSpacing(4);

    // UI behavior
#ifdef QT4
    QGroupBox *behaviorBox = new QGroupBox(tr("Behavior"), widget());
    QVBoxLayout *behaviorBoxLayout = new QVBoxLayout(behaviorBox);
#else
    QVGroupBox *behaviorBox = new QVGroupBox(tr("Behavior"), widget());
#endif

    layout->addWidget(behaviorBox);

    QWidget *hbox = new QWidget(behaviorBox);
    QHBoxLayout *hboxlayout = new QHBoxLayout(hbox);
    hboxlayout->setMargin(0);
    hboxlayout->setSpacing(4);

    QLabel *playlistAutoTriggerFilterIntervalLabel = new QLabel(tr("Auto-trigger search after:"), hbox);
    hboxlayout->addWidget(playlistAutoTriggerFilterIntervalLabel);

#ifdef QT4
    playlistAutoTriggerFilterInterval_ = new QSpinBox(hbox);
    playlistAutoTriggerFilterInterval_->setMinimum(0);
    playlistAutoTriggerFilterInterval_->setMaximum(20000);
    playlistAutoTriggerFilterInterval_->setSingleStep(50);
#else
    playlistAutoTriggerFilterInterval_ = new QSpinBox(0, 20000, 50, hbox);
#endif

    hboxlayout->addWidget(playlistAutoTriggerFilterInterval_);
    playlistAutoTriggerFilterInterval_->setSuffix(" ms");
    connect(playlistAutoTriggerFilterInterval_, SIGNAL(valueChanged(int)), this, SLOT(setPlaylistAutoTriggerFilterInterval(int)));

    changePlayInfoLayoutInPortaitMode_ = new QCheckBox(tr("Adjust the play info layout in portrait mode"), behaviorBox);
    connect(changePlayInfoLayoutInPortaitMode_, SIGNAL(toggled(bool)),
            this, SLOT(setChangePlayInfoLayoutInPortaitMode(bool)));

    touchModeEnabled_ = new QCheckBox(tr("Optimized Touch/Pen mode"), behaviorBox);
    connect(touchModeEnabled_, SIGNAL(toggled(bool)),
            this, SLOT(setTouchModeEnabled(bool)));

    kineticScrollingEnabled_ = new QCheckBox(tr("Smooth Kinetic Scrolling"), behaviorBox);
    connect(kineticScrollingEnabled_, SIGNAL(toggled(bool)),
            this, SLOT(setKineticScrollingEnabled(bool)));

#ifdef QT4
    behaviorBoxLayout->addWidget(hbox);
    behaviorBoxLayout->addWidget(changePlayInfoLayoutInPortaitMode_);
    behaviorBoxLayout->addWidget(touchModeEnabled_);
    behaviorBoxLayout->addWidget(kineticScrollingEnabled_);
#endif

    // Skin
#ifdef QT4
    QGroupBox *skinBox = new QGroupBox(tr("Skin"), widget());
    QVBoxLayout *skinBoxLayout = new QVBoxLayout(skinBox);
#else
    QVGroupBox *skinBox = new QVGroupBox(tr("Skin"), widget());
#endif

    layout->addWidget(skinBox);

    // skin selector
    QLabel *skinLabel1 = new QLabel(tr("Quickswitch Skin 1 (default): "), skinBox);
    skinList1 = new QComboBox(skinBox);
    connect(skinList1, SIGNAL(activated(const QString&)), this, SLOT(setSkin1(const QString&)));

    QLabel *skinLabel2 = new QLabel(tr("Quickswitch Skin 2: "), skinBox);
    skinList2 = new QComboBox(skinBox);
    connect(skinList2, SIGNAL(activated(const QString&)), this, SLOT(setSkin2(const QString&)));

    SkinManager *skinManager = new SkinManager(qGlobalConfig.skinPath(), this);
#ifdef QT4
    skinList1->addItems(skinManager->skinList());
    skinList2->addItems(skinManager->skinList());
#else
    skinList1->insertStringList(skinManager->skinList());
    skinList2->insertStringList(skinManager->skinList());
#endif
    delete skinManager;

#ifdef QT4
    skinBoxLayout->addWidget(skinLabel1);
    skinBoxLayout->addWidget(skinList1);
    skinBoxLayout->addWidget(skinLabel2);
    skinBoxLayout->addWidget(skinList2);
#endif
}
